function Converter_MicroSim2KiCad_IHM_Main
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Function: Converter_MicroSim2KiCad_IHM_Main
%Autor: Daniel lvarez-Campana
%Entradas: None
%Salidas: None
%Comentarios: Interfaz principal del proyecto
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Creacin de la ruta cache
system('mkdir C:\CacheMatlab');
system('setx MCR_CACHE_ROOT C:\CacheMatlab');
system('cd C:\CacheMatlab');
%% Inicializacin
proj_fullpath = mfilename('fullpath');
[projPath,~,~]= fileparts (proj_fullpath);
addpath(genpath(projPath));

logos=load([projPath '\_Libs\' 'logos.mat']);

defaultColor=[0.55 0.6 0.85];

% Comprobar si ya existe la Interfaz
IHM_Handles = get(0,'Children');
for ii=1:size(IHM_Handles)
    check = IHM_Handles(ii);
    if strcmp(check.Name,'Microsim to KiCad Converter')==1
        figure(check);
        return
    end
end

% Ajustar tamao de la ventana
screenSize = get(groot,'ScreenSize');
screenWidth = screenSize(3);
screenHeight = screenSize(4);
ScreenRatio = screenHeight/screenWidth;
IHM_main_position =[0.05 0.05 0.90 0.90];

%% Crear Interfaz Principal
IHM_main = figure('Units','normalized',...
    'Name','Microsim to KiCad Converter',...
    'Numbertitle','off',...
    'MenuBar','None',...
    'Color', defaultColor,...
    'Resize','off',...
    'DockControls','off',...
    'OuterPosition',IHM_main_position);

%% Panel: Logos
panelLogoUVa = uipanel('Parent',IHM_main,...
    'Title','',...
    'BorderType','None',...
    'Units','normalized',...
    'BackgroundColor',defaultColor,...
    'Position',[0.75 0.85 0.125 0.14]);
axeLogoUVa = axes('Parent',panelLogoUVa,...
    'Units','normalized',...
    'Xcolor',[1,1,1],...
    'Ycolor',[1,1,1],...
    'Position',[0 0 1 1]);
imshow(logos.logoUVA,'Parent',axeLogoUVa);

panelLogoEII = uipanel('Parent',IHM_main,...
    'Title','',...
    'BorderType','None',...
    'Units','normalized',...
    'BackgroundColor',defaultColor,...
    'Position',[0.902 0.85 0.08 0.14]);
axeLogoEII = axes('Parent',panelLogoEII,...
    'Units','normalized',...
    'Xcolor',[1,1,1],...
    'Ycolor',[1,1,1],...
    'Position',[0 0 1 1]);
imshow(logos.logoEII,'Parent',axeLogoEII);

%% Barra de Menu: Opciones e Idiomas
menuLang = uitoolbar('Parent',IHM_main);
buttonESP =  uitoggletool('Parent',menuLang,...
    'CData',logos.logoESP,...
    'TooltipString','Castellano',...
    'Enable','On',...
    'State','On');

buttonENG =  uitoggletool('Parent',menuLang,...
    'CData',logos.logoENG,...
    'TooltipString','English',...
    'Enable','On',...
    'State','Off');

menucomponent = uimenu('Label',dicLang('@ajustes',buttonESP.State,buttonENG.State));
addnewcomponent = uimenu(menucomponent,'Label',dicLang('@addcomponent',buttonESP.State,buttonENG.State),...
    'Callback', @AddComponent);
openlibraryfolder = uimenu(menucomponent,'Label',dicLang('@openlibraryfolder',buttonESP.State,buttonENG.State),...
    'Callback', @OpenLibrary);
menuhelp = uimenu('Label',dicLang('@help',buttonESP.State,buttonENG.State));
manualusuario = uimenu(menuhelp,'Label',dicLang('@userguide',buttonESP.State,buttonENG.State));

set(addnewcomponent,'CallBack',@(src,evt) AddComponent(IHM_main,buttonESP,buttonENG));
set(openlibraryfolder,'CallBack',@(src,evt) OpenLibrary);
set(manualusuario,'CallBack',@(src,evt) OpenUserGuide);
set(buttonESP,'ClickedCallBack',@(src,evt) cambioLang(IHM_main,buttonESP,buttonENG,'ESP'));
set(buttonENG,'ClickedCallBack',@(src,evt) cambioLang(IHM_main,buttonESP,buttonENG,'ENG'));

%% Panel: Cargar archivo
panelLoadFile = uipanel('Title',dicLang('@panelFileSelection',buttonESP.State,buttonENG.State),...
    'Units','normalized',...
    'FontSize',10,...
    'BackgroundColor',defaultColor,...
    'FontWeight','Bold',...
    'Position',[0.02 0.85 0.7 0.15]);

textFile = uicontrol(panelLoadFile,...
    'Style','text',...
    'Units','normalized',...
    'HorizontalAlignment','Left',...
    'FontSize',10,...
    'String','',...
    'Position',[0.28 0.35 0.7 0.3]);

bottonLoadFile = uicontrol('Parent',panelLoadFile,...
    'Style','PushButton',...
    'Units','normalized',...
    'FontSize',10,...
    'String',dicLang('@buttonLoadFile',buttonESP.State,buttonENG.State),...
    'Position',[0.02 0.1 0.23 0.35],...
    'Enable','Off',...
    'Callback', @LoadFile);

bottonSelectLoadFile = uicontrol('Parent',panelLoadFile,...
    'Style','PushButton',...
    'Units','normalized',...
    'FontSize',10,...
    'String',dicLang('@buttonFileSelection',buttonESP.State,buttonENG.State),...
    'Position',[0.02 0.6 0.23 0.35]);
set(bottonSelectLoadFile,'Callback',@(src,evt) SelectFile(textFile,bottonLoadFile,'Load',dicLang('@optionLoad',buttonESP,buttonENG)))
datosPrograma = getappdata(IHM_main,'datosPrograma');
%% Panel: Configuracin
panelSettings = uipanel('Title',dicLang('@panelConfig',buttonESP.State,buttonENG.State),...
    'Units','normalized',...
    'FontSize',10,...
    'FontWeight','Bold',...
    'BackgroundColor',defaultColor,...
    'Position',[0.02 0.18 0.96 0.66]);

textComponentes = uicontrol('Parent',panelSettings,...
    'Style','text',...
    'Units','normalized',...
    'FontSize',11,...
    'String',dicLang('@textComponents',buttonESP.State,buttonENG.State),...
    'Position',[0.05 0.87 0.3 0.05]);

listComponentes = uicontrol('Parent',panelSettings,...
    'Style','listbox',...
    'Units','normalized',...
    'String','...',...
    'Position',[0.05 0.05 0.3 0.8],...
    'Callback', @ActualizarModelos);

textModelos = uicontrol('Parent',panelSettings,...
    'Style','text',...
    'Units','normalized',...
    'FontSize',11,...
    'String',dicLang('@textModels',buttonESP.State,buttonENG.State),...
    'Position',[0.4 0.87 0.3 0.05]);

listModelos = uicontrol('Parent',panelSettings,...
    'Style','listbox',...
    'Units','normalized',...
    'String','...',...
    'Position',[0.4 0.05 0.3 0.8],...
    'Callback', @SeleccionModelos);

textRotacion = uicontrol('Parent',panelSettings,...
    'Style','text',...
    'Units','normalized',...
    'String',dicLang('@textRotation',buttonESP.State,buttonENG.State),...
    'Position',[0.75 0.87 0.15 0.05]);

bottonRotacion = uicontrol('Parent',panelSettings,...
    'Style','popupmenu',...
    'Units','normalized',...
    'String', {'0';'90';'180';'-90'},...
    'Position',[0.75 0.55 0.15 0.3]);

bottonAplicarRotacion = uicontrol('Parent',panelSettings,...
    'Style','PushButton',...
    'Units','normalized',...
    'FontSize',10,...
    'String',dicLang('@buttonAplicarRotacion',buttonESP.State,buttonENG.State),...
    'Position',[0.75 0.72 0.15 0.05],...
    'Enable','Off',...
    'Callback', @RotacionPermanente);

bottonAplicarRotacionLibreria = uicontrol('Parent',panelSettings,...
    'Style','PushButton',...
    'Units','normalized',...
    'FontSize',10,...
    'String',dicLang('@buttonAplicarRotacionLibreria',buttonESP.State,buttonENG.State),...
    'Enable','Off',...
    'Position',[0.75 0.65 0.15 0.05]);

panelPrevisu =uipanel('Parent',panelSettings,...
    'Title','',...
    'Visible','off',...
    'Units','normalized',...
    'FontSize',10,...
    'FontWeight','Bold',...
    'BackgroundColor',[1 1 1],...
    'Position',[0.75 0.25 0.15 0.4]);

axePrevisuModel = axes('Parent',panelPrevisu,...
    'Units','normalized',...
    'Xcolor',[1,1,1],...
    'Ycolor',[1,1,1],...
    'Position',[0 0 1 1]);

set(listComponentes, 'Callback', @(src,evt) ActualizarModelos(IHM_main,listComponentes,listModelos));
set(listModelos, 'Callback', @(src,evt) SeleccionModelos(IHM_main,listComponentes,listModelos));
set(bottonAplicarRotacion, 'Callback', @(src,evt) Rotacion(IHM_main,listComponentes.String(listComponentes.Value), bottonRotacion.String(bottonRotacion.Value)));
set(bottonAplicarRotacionLibreria, 'Callback', @(src,evt) RotacionPermanente(IHM_main,listModelos, bottonRotacion.String(bottonRotacion.Value)));

%% Panel: Guardar archivo
panelSaveFile = uipanel('Title',dicLang('@panelFileSaving',buttonESP.State,buttonENG.State),...
    'Units','normalized',...
    'FontSize',10,...
    'BackgroundColor',defaultColor,...
    'FontWeight','Bold',...
    'Position',[0.02 0.02 0.96 0.15]);

textSaveFile = uicontrol('Parent',panelSaveFile,...
    'Style','text',...
    'Units','normalized',...
    'HorizontalAlignment','Left',...
    'FontSize',10,...
    'String','',...
    'Position',[0.168 0.3 0.6 0.3]);

bottonSaveFile = uicontrol('Parent',panelSaveFile,...
    'Style','PushButton',...
    'Units','normalized',...
    'FontSize',10,...
    'String',dicLang('@buttonFileSaving',buttonESP.State,buttonENG.State),...
    'Position',[0.014 0.1 0.138 0.35],...
    'Enable','Off',...
    'Callback', @SaveFile);

bottonSelectSaveFile = uicontrol('Parent',panelSaveFile,...
    'Style','PushButton',...
    'Units','normalized',...
    'String',dicLang('@buttonSelectSaving',buttonESP.State,buttonENG.State),...
    'FontSize',10,...
    'Position',[0.014 0.6 0.138 0.35],...
    'Enable','Off');

set(bottonSelectSaveFile,'Callback',@(src,evt) SelectFile(textSaveFile,bottonSaveFile,'Save',dicLang('@optionSave',buttonESP.State,buttonENG.State), textFile.String));
set(bottonLoadFile, 'Callback', @(src,evt) LoadFile(IHM_main,textFile.String,listComponentes,listModelos, bottonSelectSaveFile,bottonAplicarRotacion,bottonAplicarRotacionLibreria));
set(bottonSaveFile, 'Callback', @(src,evt) SaveFile(IHM_main,textSaveFile.String,dicLang('@ArchivoGuardado',buttonESP.State,buttonENG.State)));
end
