function LoadFile( varargin )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Function: LoadFile
%Autor: Daniel lvarez-Campana
%Entradas: 
%         Argumento1 = Handle de la interfaz principal
%         Argumento2 = Ruta del archivo a cargar
%         Argumento3 = Handle de la lista de componentes
%         Argumento4 = Handle de la lista de modelos
%         Argumento5 = Handle del boton de guardar archivo
%         Argumento6 = Handle del boton de rotar componente
%         Argumento7 = Handle del boton de rotacion permanente
%Salidas: None
%Comentarios: Funcin para leer y almacenar los datos necesarios del .pca
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Lectura de argumentos de entrada
IHM_main = varargin{1};
rutaCarga = varargin{2};
listaComponentes = varargin{3};
listaModelos = varargin{4};
bottonSelectSaveFile = varargin{5};
bottonAplicarRotacion = varargin{6};
bottonAplicarRotacionLibreria = varargin{7};
%% Carga de fichero de correspondencias
%Obtener ruta de correspondencia a partir de la libreria
libCorresp = '_Libs\Correspondencias.txt';
rutaLlamada = which('Converter_MicroSim2KiCad_IHM_Main.m');
delimiter = find (rutaLlamada=='\');
rutaCorresp = [rutaLlamada(1:delimiter(end-1)) libCorresp];

%Apertura de correspondencia
[fileCorrespond,errmsg] = fopen(rutaCorresp);

%Gestion de error de apertura
if fileCorrespond < 1
    msgbox(errmsg)
    return
end

%Lectura de contenido y cierre de fichero
textContMicroSM=textscan(fileCorrespond, '%s', 'Delimiter', '\n');
fclose('all');

%Gestion de datos obtenidos del fichero de correspondencia
textContMicroSM=textContMicroSM{1,1};

for x=1:size(textContMicroSM,1)
    if ~isempty(textContMicroSM{x,1})
        datosCorresp = textscan(textContMicroSM{x,1}, '%s');
        datosCorresp = datosCorresp{1,1};
        dict(x,1) = datosCorresp(1);
        dict(x,2) = datosCorresp(2);
        dict(x,3) = datosCorresp(3);
    end
end

%% Carga de fichero que se va a convertir de MicroSim
%Obtener ruta de fichero a convertir de microsim
[fileID,errmsg] = fopen(rutaCarga);

%Gestion de error de apertura
if fileID < 1
    msgbox(errmsg)
    return
end

% Lectura de contenido y cierre de fichero de MicroSim
textContMicroSM=textscan(fileID, '%s', 'Delimiter', '\n');
fclose('all');

textContMicroSM=textContMicroSM{1,1};

%% Gestion de elementos obtenidos de MicroSim
% Borde de placa encontradas
Index_bordeplaca = find(contains(textContMicroSM,'*line BoardOutline'));
% Lineas encontradas
Index_linea = find(contains(textContMicroSM,'*line'));
% Arcos encontrados
Index_arco = find(contains(textContMicroSM,'*arc'));
% Pasantes encontradas
Index_hole = find(contains(textContMicroSM,'*hole'));
% Textos encontrados
Index_texto = find(contains(textContMicroSM,'*text'));
% Componentes encontrados
Index_componentes = find(contains(textContMicroSM,'*component'));
% Rutas encontradas
Index_segment = find(contains(textContMicroSM,'@seg'));
% Vias encontradas
Index_vias = find(contains(textContMicroSM,'@via'));
 
% Recuperacin de borde de placa encontrada
if ~isempty(Index_bordeplaca)
    for x = 1:length(Index_bordeplaca)
        datos_bordeplaca = textscan(textContMicroSM{Index_bordeplaca(x),1}, '%s');
        datos_bordeplaca = datos_bordeplaca{1,1};
        for y = 1:5
            %[origenX, origenY, desplX, desplY, grosor]
            bordeplaca(x,y) = str2num(cell2mat(datos_bordeplaca(y+2)));
        end
    end
    existePlaca = 1;
else
    % Mensaje de error si no se encuentra una placa definida
    warndlg(dicLang('@nobordesplaca',buttonESP.State,buttonENG.State));
    return;
end

% Calculo del ancho y alto de la placa
if existePlaca
    xmin = min(bordeplaca(:,1));
    xmax = max(bordeplaca(:,1));
    ymin = min(bordeplaca(:,2));
    ymax = max(bordeplaca(:,2));
    ancho = xmax - xmin;
    alto = ymax - ymin;
end

% Recuperacin de lineas encontradas
if ~isempty(Index_linea)
    for x = 1:length(Index_linea)
        datos_linea = textscan(textContMicroSM{Index_linea(x),1}, '%s');
        datos_linea = datos_linea{1,1};
        for y = 1:5
            %[origenX, origenY, desplX, desplY, grosor]
            lineas(x,y) = str2num(cell2mat(datos_linea(y+2)));
        end
        % Capa donde se encuentra cada lnea
        if strcmp(datos_linea(2),'SilkTop')
            lineas(x,6) = 0;
        elseif strcmp(datos_linea(2),'SilkBot')
            lineas(x,6) = 2;
        elseif strcmp(datos_linea(2),'Solder')
            lineas(x,6) = 1;
        else
            lineas(x,6) = 3;
        end
    end
    existeLinea = 1;
else
    existeLinea = 0;
    lineas_kicad = {};
end

% Recuperacin de arcos encontrados
if ~isempty(Index_arco)
    for x = 1:length(Index_arco)
        datos_arco = textscan(textContMicroSM{Index_arco(x),1}, '%s');
        datos_arco = datos_arco{1,1};
        for y = 1:6
            %[origenX, origenY, radio, angulo origen, angulo final, grosor]
            arcos(x,y) = str2num(cell2mat(datos_arco(y+2)));
        end
    end
    existeArco = 1;
else
    existeArco = 0;
    arcos_kicad = {};
end

% Recuperacin de pasantes encontrados
if ~isempty(Index_hole)
    for x = 1:length(Index_hole)
        datos_hole = textscan(textContMicroSM{Index_hole(x),1}, '%s');
        datos_hole = datos_hole{1,1};
        for y = 1:3
            %[origenX, origenY, radio]
            pasantes(x,y) = str2num(cell2mat(datos_hole(y+3)));
        end
    end
    existepasante = 1;
else
    existepasante = 0;
    pasantes = {};
end

% Recuperacin de textos encontrados
if ~isempty(Index_texto)
    for x = 1:length(Index_texto)
        datos_linea1 = textContMicroSM{Index_texto(x)+1,1};
        % texto
        letras_texto(x) = string(datos_linea1(17:end));
        datos_linea2 = textscan(textContMicroSM{Index_texto(x)+2,1}, '%s');
        datos_linea2 = datos_linea2{1,1};
        for y = 1:6
            %[origenX, origenY, alineado, angulo, tamaoletra, grosor]
            datos_texto(x,y) = str2num(cell2mat(datos_linea2(y+3)));
        end
        % Capa donde se encuentra
        if strcmp(datos_linea2(3),'SilkTop')
            datos_texto(x,7) = 0;
        elseif strcmp(datos_linea2(3),'SilkBot')
            datos_texto(x,7) = 2;
        else 
            datos_texto(x,7) = 1;
        end
    end
    existeTexto = 1;
else
    existeTexto = 0;
    letras_texto = {};
    datos_texto = {};
end

% Recuperacin de componentes encontrados
existePin = 0;
if ~isempty(Index_componentes)
    for x = 1:length(Index_componentes)
        componente = textscan(textContMicroSM{Index_componentes(x),1}, '%s');
        componente = componente{1,1};
        % Nombre componente
        nombre_componente(x,1) = string(componente(3));
        % Nombre de Referencia
        nombre_componente(x,2) = string(componente(4));
        for y = 1:3
            %[origenX, origenY, angulo]
            datos_componente(x,y) = str2num(cell2mat(componente(y+4)));
        end
        % Recuperacin de datos del nombre de Referencia
        for y=1:8
            referencia = textscan(textContMicroSM{Index_componentes(x)+y,1}, '%s');
            referencia=referencia{1,1};
            if strcmp(referencia(1), 'display') && strcmp(referencia(2), '[SilkTop]')
                % [origenX, origenY, angulo, tamaoletra, grosor]
                datos_ref(x,1)=str2double(referencia(4));
                datos_ref(x,2)=str2double(referencia(5));
                datos_ref(x,3)=str2double(referencia(7));
                datos_ref(x,4)=str2double(referencia(8));
                datos_ref(x,5)=str2double(referencia(9));
                break;
            end
        end
        % Recuperacin datos Pads
        index_footprint = find(contains(textContMicroSM, strcat('*footprint', string(' '),nombre_componente(x))));
        textMicroCortado = textContMicroSM(index_footprint:end);
        index_pin = find(contains(textMicroCortado,'@pin'));
        for y = 1:(index_pin(2)-(index_pin(1)+1))
            pin = textscan(textMicroCortado{index_pin(1)+y,1}, '%s');
            pin = pin{1,1};
            if strcmp(pin(1),'pin') == 1
                if existePin == 1
                    datos_pin(end+1,1) = nombre_componente(x,1); %nombre componente
                    datos_pin(end,2) = nombre_componente(x,2); % referencia componente
                    datos_pin(end,3) = pin(2); %numero de pin
                    datos_pin(end,4) = cell2mat(pin(3)); %posx
                    datos_pin(end,5) = cell2mat(pin(4)); %posy
                    datos_pin(end,6) = cell2mat(pin(5)); %angulo
                    datos_pin(end,7) = pin(end); % THT or SMD
                    tipo_pin = pin(6);
                    tipo_pin = strrep(tipo_pin, 'x','-');
                    tipo_pin = strsplit(string(tipo_pin),'-');
                    datos_pin(end,8) = tipo_pin(1); % tipo pad
                    datos_pin(end,9) = tipo_pin(2); % medida pad x
                    if strcmp(pin(end), 'thru')  % THT
                        if strcmp(tipo_pin(1), 'rect') ||  strcmp(tipo_pin(1), 'oval')
                            datos_pin(end,10) = tipo_pin(3);    % medida pad y
                            datos_pin(end,11) = tipo_pin(4);    % medida radio interno
                        else
                            datos_pin(end,10) = tipo_pin(2);
                            datos_pin(end,11) = tipo_pin(3);
                        end
                    else   % SMD
                        if strcmp(tipo_pin(1), 'rect') ||  strcmp(tipo_pin(1), 'oval')
                            datos_pin(end,10) = tipo_pin(3);    % medida pad y
                            datos_pin(end,11) = '000';    % medida radio interno
                        else
                            datos_pin(end,10) = tipo_pin(2);
                            datos_pin(end,11) = '000';
                        end
                    end
                else
                   datos_pin(1,1) = nombre_componente(x,1); %nombre componente
                   datos_pin(1,2) = nombre_componente(x,2); % referencia componente
                   datos_pin(1,3) = pin(2); %numero de pin
                   datos_pin(1,4) = cell2mat(pin(3)); %posx
                   datos_pin(1,5) = cell2mat(pin(4)); %posy
                   datos_pin(1,6) = cell2mat(pin(5)); %angulo
                   datos_pin(end,7) = pin(end);  % THT or SMD   
                   tipo_pin = pin(6);
                   tipo_pin = strrep(tipo_pin, 'x','-');
                   tipo_pin = strsplit(string(tipo_pin),'-');
                   datos_pin(1,8) = tipo_pin(1);    % tipo pad
                   datos_pin(1,9) = tipo_pin(2);    % medidas pad
                   datos_pin(1,10) = tipo_pin(3);
                   if strcmp(pin(end), 'thru')  % THT
                        if strcmp(tipo_pin(1), 'rect') ||  strcmp(tipo_pin(1), 'oval')
                            datos_pin(1,10) = tipo_pin(3);    % medida pad y
                            datos_pin(1,11) = tipo_pin(4);    % medida radio interno
                        else
                            datos_pin(1,10) = tipo_pin(2);
                            datos_pin(1,11) = tipo_pin(3);
                        end
                    else   % SMD
                        if strcmp(tipo_pin(1), 'rect') ||  strcmp(tipo_pin(1), 'oval')
                            datos_pin(1,10) = tipo_pin(3);    % medida pad y
                            datos_pin(1,11) = '000';    % medida radio interno
                        else
                            datos_pin(1,10) = tipo_pin(2);
                            datos_pin(1,11) = '000';
                        end
                    end 
                   existePin = 1;
                end
            else
                break;
            end
        end
    end
    existeComponente = 1;
else
    existeComponente = 0;
    nombre_componente = {};
    datos_componente = {};
    datos_pin = {};
    datos_ref = {};
end

% Recuperacin de pistas encontradas
if ~isempty(Index_segment)
    for x = 1:length(Index_segment)
        datos_ruta = textscan(textContMicroSM{Index_segment(x),1}, '%s');
        datos_ruta = datos_ruta{1,1};
        for y = 1:5
            %[origenX, origenY, desplX, desplY, grosor]
            rutas(x,y) = str2num(cell2mat(datos_ruta(y+2)));            
        end
        if strcmp(datos_ruta(2),'Solder') == 1
            % cara inferior
            rutas(x,6) = 1;
        else
            % cara superior
            rutas(x,6) = 0;
        end
    end
    existeRuta = 1;
else
    existeRuta = 0;
    rutas = {};
end

% Recuperacin de vas encontradas
if ~isempty(Index_vias)
    for x = 1:length(Index_vias)
        datos_via = textscan(textContMicroSM{Index_vias(x),1}, '%s');
        datos_via = datos_via{1,1};
        % [origenX, origenY]
        vias(x,1) = str2double(datos_via(4));
        vias(x,2) = str2double(datos_via(5));
        tipo_via = strrep(datos_via(6), 'x','-');
        tipo_via = strsplit(string(tipo_via),'-');
        for y=1:2
            % [Radio externo, Radio interno]
            medida_via = char(tipo_via(y+1));
            vias(x,2+y) = str2double([medida_via(1:end-2) '.' medida_via(end-1:end)]);
        end
    end
    existeVia = 1;
else
    existeVia = 0;
    vias = {};
end

%% Tratamiento de elementos encontrados
% Lineas: Cambio de escalas Inch a mm y cambio de coordenadas
if existeLinea
    lineas_kicad(:,1) = 25.4*lineas(:,1)+240-25.4*0.5*ancho;
    lineas_kicad(:,2) = 190-25.4*lineas(:,2)+25.4*0.5*alto;
    lineas_kicad(:,3) = 25.4*lineas(:,3)+240-25.4*0.5*ancho;
    lineas_kicad(:,4) = 190-25.4*lineas(:,4)+25.4*0.5*alto;
    lineas_kicad(:,5) = 25.4*lineas(:,5);
    lineas_kicad(:,6) = lineas(:,6);
end
% Arcos: Cambio de escalas Inch a mm, cambio de coordenadas y cambio geometricos
if existeArco
    arcos_kicad(:,1) = 25.4*arcos(:,1)+240-25.4*0.5*ancho;
    arcos_kicad(:,2) = 190-25.4*arcos(:,2)+25.4*0.5*alto;
    arcos_kicad(:,3) = 25.4*(arcos(:,1) + 25.4*arcos(:,3).*cos(arcos(:,4)))+240-25.4*0.5*ancho;
    arcos_kicad(:,4) = 25.4*(arcos(:,2) + 25.4*arcos(:,3).*cos(arcos(:,4)))+25.4*0.5*alto;
    arcos_kicad(:,5) = arcos(:,4)-arcos(:,3);
    arcos_kicad(:,6) = 25.4*arcos(:,6);
end
% Borde: Cambio de escalas Inch a mm y cambio de coordenadas
if existePlaca
    borde_kicad(:,1) = 25.4*bordeplaca(:,1)+240-25.4*0.5*ancho;
    borde_kicad(:,2) = 190-25.4*bordeplaca(:,2)+25.4*0.5*alto;
    borde_kicad(:,3) = 25.4*bordeplaca(:,3)+240-25.4*0.5*ancho;
    borde_kicad(:,4) = 190-25.4*bordeplaca(:,4)+25.4*0.5*alto;
    borde_kicad(:,5) = 25.4*bordeplaca(:,5);
end
% Texto: Cambio de escalas Inch a mm y cambio de coordenadas 
if existeTexto
    datos_texto(:,1) = 25.4*datos_texto(:,1)+240-25.4*0.5*ancho;
    datos_texto(:,2) = 190-25.4*datos_texto(:,2)+25.4*0.5*alto;
    datos_texto(:,5) = 25.4*datos_texto(:,5);
    datos_texto(:,6) = 25.4*datos_texto(:,6);
end
% Componentes: Cambio de escalas Inch a mm y cambio de coordenadas
if existeComponente
    datos_componente(:,1) = 25.4*datos_componente(:,1)+240-25.4*0.5*ancho;
    datos_componente(:,2) = 190-25.4*datos_componente(:,2)+25.4*0.5*alto;
end
% Pin: Cambio de escalas Inch a mm y cambio de coordenadas
if existePin
    datos_pin(:,4) = 25.4*str2double(datos_pin(:,4));
    datos_pin(:,5) = 25.4*str2double(datos_pin(:,5));
    for x=1:size(datos_pin,1)
        for y=9:11
            pin = char(datos_pin(x,y));
            datos_pin(x,y) = str2mat([pin(1:end-2) '.' pin(end-1:end)]);
        end
    end
end
% Pistas: Cambio de escalas Inch a mm y cambio de coordenadas
if existeRuta
    rutas(:,1) = 25.4*rutas(:,1)+240-25.4*0.5*ancho;
    rutas(:,2) = 190-25.4*rutas(:,2)+25.4*0.5*alto;
    rutas(:,3) = 25.4*rutas(:,3)+240-25.4*0.5*ancho;
    rutas(:,4) = 190-25.4*rutas(:,4)+25.4*0.5*alto;
    rutas(:,5) = 25.4*rutas(:,5);
end
% Pasantes: Cambio de escalas Inch a mm y cambio de coordenadas
if existepasante
    pasantes(:,1) = 25.4*pasantes(:,1)+240-25.4*0.5*ancho;
    pasantes(:,2) = 190-25.4*pasantes(:,2)+25.4*0.5*alto;
    pasantes(:,3) = 25.4*pasantes(:,3);
end
% Vas: Cambio de escalas Inch a mm y cambio de coordenadas
if existeVia
    vias(:,1) = 25.4*vias(:,1)+240-25.4*0.5*ancho;
    vias(:,2) = 190-25.4*vias(:,2)+25.4*0.5*alto;
end
% Referencia: Cambio de escalas Inch a mm y cambio de coordenadas
if existeComponente
    datos_ref(:,1) = 25.4*datos_ref(:,1);
    datos_ref(:,2) = -25.4*datos_ref(:,2);
    datos_ref(:,4) = 25.4*datos_ref(:,4);
    datos_ref(:,5) = 25.4*datos_ref(:,5);
end


%% Busqueda de componentes en librera
elements=1;
componentes = unique(nombre_componente(:,1));
for x= 1:size(componentes)
    Index_Comp = find(contains(dict(:,1), componentes(x)));
    if ~isempty(Index_Comp)
        for xx= 1:size(Index_Comp)
            if strcmp(dict(Index_Comp(xx),1),componentes(x))
                %[nombre componente MicroSim, nombre componente KiCad, angulo, modelo si/no]
                correspondencias(elements,1)=componentes(x);
                correspondencias(elements,2)=dict(Index_Comp(xx),2);
                correspondencias(elements,3)=dict(Index_Comp(xx),3);
                if xx == 1
                    correspondencias(elements,4) = 1;
                else
                    correspondencias(elements,4) = 0;
                end
                elements=elements+1;
            end
        end
    end
end
% Lista de componentes
componentesencontrados = unique(correspondencias(:,1));
% Componentes no encontrados
componentesNOencontrados = setdiff(componentes,componentesencontrados);
if ~isempty(componentesNOencontrados)
    msgbox(sprintf('Los siguientes componentes no se han encotrado en la librera:\n%s',strjoin(componentesNOencontrados, '\n')));
end
Index_primercomponente = find(contains(correspondencias(:,1), componentesencontrados(1)));
element = 1;
% Lista de modelos
for x=1:size(Index_primercomponente)
    if strcmp(correspondencias(Index_primercomponente(x),1),componentesencontrados(1))
        modelosasociados(element) = correspondencias(Index_primercomponente(x),2);
        element = element+1;
    end
end

%% Creamos listas
set(listaComponentes,'String',componentesencontrados);
set(listaModelos,'String',modelosasociados);
set(listaComponentes,'Value',1);
set(listaModelos,'Value',1);

%% Guardado de datos utiles para usarlo ms tarde
datosPrograma = {lineas_kicad;borde_kicad;letras_texto;datos_texto;nombre_componente; datos_componente;correspondencias; datos_pin; pasantes; rutas; arcos_kicad; vias; datos_ref};
setappdata(IHM_main,'datosPrograma',datosPrograma);
%% Habilitado de funciones de rotacion y seleccion de archivo de guardado
set(bottonSelectSaveFile, 'Enable', 'On');
set(bottonAplicarRotacion, 'Enable', 'On');
set(bottonAplicarRotacionLibreria, 'Enable', 'On');

end

