function SaveFile(varargin)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Function: SaveFile
%Autor: Daniel lvarez-Campana
%Entradas:
%         Argumento1 = Handle del interfaz principal
%         Argumento2 = Ruta de guardado
%         Argumento3 = Mensaje de finalizacin
%Salidas: None
%Comentarios: Funcin para crear el archivo .kicad_pcb con toda la 
%          informacin del archivo .pca
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Lectura de argumentos de entrada
IHM_main = varargin{1};
rutaGuardado = varargin{2};
msg = varargin{3};

%% Desglose de los datos almacenados
datosPrograma = getappdata(IHM_main,'datosPrograma');
lineas_kicad = datosPrograma(1);
lineas_kicad = lineas_kicad{1,1};
borde_kicad = datosPrograma(2);
borde_kicad = borde_kicad{1,1};
letras_texto = datosPrograma(3);
letras_texto = letras_texto{1,1};
datos_texto = datosPrograma(4);
datos_texto = datos_texto{1,1};
nombre_componente = datosPrograma(5);
nombre_componente = nombre_componente{1,1};
datos_componente = datosPrograma(6);
datos_componente = datos_componente{1,1};
correspondencias = datosPrograma(7);
correspondencias = correspondencias{1,1};
datos_pin = datosPrograma(8);
datos_pin = datos_pin{1,1};
pasantes = datosPrograma(9);
pasantes = pasantes{1,1};
rutas_kicad = datosPrograma(10);
rutas_kicad = rutas_kicad{1,1};
arcos = datosPrograma(11);
arcos = arcos{1,1};
vias = datosPrograma(12);
vias = vias{1,1};
datos_ref = datosPrograma(13);
datos_ref = string(datos_ref{1,1});

%% Obtenencin de ruta de libreria de componentes
libcomponentes = '_Libs\';
rutaLlamada = which('Converter_MicroSim2KiCad_IHM_Main.m');
delimiter = find (rutaLlamada=='\');
rutalibcomponentes = [rutaLlamada(1:delimiter(end-1)) libcomponentes];
                    
%% Obtenencin de ruta de encabezado a partir de la libreria
libEncabezado = '_Code\_Libs\plantilla.txt';
rutaEncabezado = [rutaLlamada(1:delimiter(end-1)) libEncabezado];

%% Creacin del archivo
% Copiamos nuestro encabezado y lo abrimos
copyfile(rutaEncabezado, rutaGuardado);

[fileID,errmsg] = fopen(rutaGuardado,'a');

if fileID < 1
    msgbox(errmsg)
    return
end

% Escribimos los bordes de la placa
if exist('borde_kicad','var') == 1
    for x= 1:size(borde_kicad,1)
        fprintf(fileID, '\n(gr_line (start %d %d) (end %d %d) (layer Edge.Cuts) (width %d))', borde_kicad(x,1), borde_kicad(x,2), borde_kicad(x,3), borde_kicad(x,4), borde_kicad(x,5));
    end
end
   
% Escribimos las lneas      
if exist('lineas_kicad','var') == 1
	for x= 1:size(lineas_kicad,1)
        if lineas_kicad(x,6) == 1
            fprintf(fileID, '\n(gr_line (start %d %d) (end %d %d) (layer B.Cu) (width %d))', lineas_kicad(x,1), lineas_kicad(x,2), lineas_kicad(x,3), lineas_kicad(x,4),lineas_kicad(x,5));
        elseif lineas_kicad(x,6) == 2
            fprintf(fileID, '\n(gr_line (start %d %d) (end %d %d) (layer B.SilkS) (width %d))', lineas_kicad(x,1), lineas_kicad(x,2), lineas_kicad(x,3), lineas_kicad(x,4),lineas_kicad(x,5));
        elseif lineas_kicad(x,6) == 0
            fprintf(fileID, '\n(gr_line (start %d %d) (end %d %d) (layer F.SilkS) (width %d))', lineas_kicad(x,1), lineas_kicad(x,2), lineas_kicad(x,3), lineas_kicad(x,4),lineas_kicad(x,5));           
        end
	end
end

% Escribimos los arcos
if exist('arcos','var') == 1
	for x= 1:size(arcos,1)
        fprintf(fileID, '\n(gr_arc (start %d %d) (end %d %d) (angle %d) (layer F.SilkS) (width %d))', arcos(x,1), arcos(x,2), arcos(x,3), arcos(x,4),arcos(x,5),arcos(x,6));
	end
end

% Escribimos los textos
if exist('datos_texto','var') == 1
	for x= 1:size(datos_texto,1)
        if datos_texto(x,7)==0
            fprintf(fileID, '\n(gr_text "%s" (at %d %d %d) (layer F.SilkS) (effects (font (size %d %d) (thickness %d))))', letras_texto(x), datos_texto(x,1), datos_texto(x,2), datos_texto(x,4), datos_texto(x,5),datos_texto(x,5), datos_texto(x,6));
        elseif datos_texto(x,7)==1
            fprintf(fileID, '\n(gr_text "%s" (at %d %d %d) (layer B.Cu) (effects (font (size %d %d) (thickness %d))(justify mirror)))', letras_texto(x), datos_texto(x,1), datos_texto(x,2), datos_texto(x,4), datos_texto(x,5),datos_texto(x,5), datos_texto(x,6));
        else
            fprintf(fileID, '\n(gr_text "%s" (at %d %d %d) (layer B.SilkS) (effects (font (size %d %d) (thickness %d))))', letras_texto(x), datos_texto(x,1), datos_texto(x,2), datos_texto(x,4), datos_texto(x,5),datos_texto(x,5), datos_texto(x,6));
        end
    end

% Escribimos los taladros
if exist('pasantes','var') == 1
	for x= 1:size(pasantes,1)
        rutapasante = char(strcat(rutalibcomponentes,'\MountingHole_2.1mm.kicad_mod'));
        infopasante = fileread(rutapasante);
        infopasante = strrep(infopasante, '@posicionx', string(pasantes(x,1)));
        infopasante = strrep(infopasante, '@posiciony', string(pasantes(x,2)));
        infopasante = strrep(infopasante, '@radio', string(pasantes(x,3)));
        fprintf(fileID,'%s', infopasante);
    end
end

% Escribimos las pistas
if exist('rutas_kicad','var') == 1
	for x= 1:size(rutas_kicad,1)
        if rutas_kicad(x, 6) == 0
            fprintf(fileID, '\n(segment (start %d %d) (end %d %d) (width %d) (layer F.Cu) )', rutas_kicad(x,1), rutas_kicad(x,2), rutas_kicad(x,3), rutas_kicad(x,4),rutas_kicad(x,5));
        else
            fprintf(fileID, '\n(segment (start %d %d) (end %d %d) (width %d) (layer B.Cu) )', rutas_kicad(x,1), rutas_kicad(x,2), rutas_kicad(x,3), rutas_kicad(x,4),rutas_kicad(x,5));
        end
	end
end

% Escribimos las vias
if exist('vias','var') == 1
	for x= 1:size(vias,1)
        fprintf(fileID, '\n(via (at %d %d) (size %d) (drill %d) (layers F.Cu B.Cu) (net 0))', vias(x,1), vias(x,2), vias(x,3), vias(x,4));
	end
end

% Escribimos los componentes
if exist('datos_componente','var') == 1
	fprintf(fileID, '\n \n');
    for x= 1:size(nombre_componente,1)
        Index_correspondencias = find((correspondencias(:,1) == nombre_componente(x)));
        if isempty(Index_correspondencias) == 0
            for y=1:size(Index_correspondencias,1)
                % Buscamos el componente en la librera y modificamos datos
                if str2double(correspondencias(Index_correspondencias(y),4)) == 1
                    nombre_biblioteca = char(strcat(rutalibcomponentes,'\',correspondencias(Index_correspondencias(y),2),'.kicad_mod'));
                    todo = fileread(nombre_biblioteca);
                    
                    NL = regexp(todo, '[\n]');
                    % Aadimos la posicin y orientacin del componente
                    new = [todo(1:NL(1)-1) char(10)  '  (at @posicionx @posiciony @angulo)' todo(NL(1):end)];                    
                    new = strrep(new, '@posicionx', string(datos_componente(x,1)));
                    new = strrep(new, '@posiciony', string(datos_componente(x,2)));
                    new = strrep(new, '@angulo', string(str2double(correspondencias(Index_correspondencias(y),3))+ datos_componente(x,3)));
                    % Copiamos su referencia
                    NLnew = regexp(new, '[\n]');
                    fprintf(fileID,'%s', new(1:NLnew(4)+21));
                    % Comprobar si la referencia esta horizonta o vertical   
                    angulo1 = mod(str2double(correspondencias(Index_correspondencias(y),3))+ datos_componente(x,3)+str2double(datos_ref(x,3)),360);
                    if angulo1 == 0 || angulo1 == 180
                        datos_ref(x,3) = '0';
                    else
                        datos_ref(x,3) = '90';
                    end
                    
                    angulo2 = mod(str2double(correspondencias(Index_correspondencias(y),3)),360);
                    switch angulo2
                        case 0
                            datos_ref(x,1) = datos_ref(x,1);
                            datos_ref(x,2) = datos_ref(x,2);
                        case 90
                           a = datos_ref(x,1);
                           datos_ref(x,1) = string(-str2double(datos_ref(x,2)));
                           datos_ref(x,2) = a;
                        case 180
                            datos_ref(x,1) = datos_ref(x,1);
                            datos_ref(x,2) = datos_ref(x,2);
                        case 270
                            a=datos_ref(x,1);
                            datos_ref(x,1)= datos_ref(x,1);
                            datos_ref(x,2)= datos_ref(x,2);
                    end
                    
                    fprintf(fileID, '%s (at %s %s %s) (layer F.SilkS)\n    (effects (font (size %s %s) (thickness %s))',nombre_componente(x,2),datos_ref(x,1), datos_ref(x,2),datos_ref(x,3),datos_ref(x,4),datos_ref(x,4), datos_ref(x,5));
                        
                    % Encontramos donde estan definidos los pads
                    Index_pads = find((datos_pin(:,2) == nombre_componente(x,2)));
                    pads = datos_pin(Index_pads,:);
                    for z=1:(size(NLnew,2))-1
                        linea = new(NLnew(z):NLnew(z+1));
                        linea = split(linea);
                        if strcmp(linea(2), '(pad') == 1
                              linea_pads=z;
                              break;
                        end
                    end 
                    % Leemos los pads de la biblioteca
                    lineas_pads = new(NLnew(linea_pads)+1:NLnew(linea_pads+size(pads,1))-1); %todos los pads
                    lineas_duro = splitlines(lineas_pads);
                    datos_duro = [];
                    for z=1:size(lineas_duro,1)
                        linea_duro = split(lineas_duro(z));
                        datos_duro(z,1) = double(linea_duro(3));	%numero pin
                        datos_duro(z,2) = double(linea_duro(7));    %posicion x
                        posy = char(linea_duro(8));    %posicion y
                        if posy(end) == ')'
                            datos_duro(z,3) = double(string(posy(1:end-1)));
                        else
                            datos_duro(z,3) = double(string(posy));
                        end
                    end
                    % Comparamos los pads para colocarlos en orden
                    comp_pads = sortrows(str2double(pads(:,[3,4,5])));
                    datos_duro = sortrows(datos_duro);
                    for z=2:size(datos_duro,1)
                        if (comp_pads(z,2) > comp_pads(z,3) && datos_duro(z,2) < datos_duro(z,3)) || (comp_pads(z,2) < comp_pads(z,3) && datos_duro(z,2) > datos_duro(z,3))
                            buffer = pads(:,4);
                            pads(:,4) = pads(:,5);
                            pads(:,5) = buffer; 
                            break;
                        end
                    end
                    
                    for z=1:size(pads,1)
                        if double(datos_duro(z,3)) < 0
                            index_pincambio = find((pads(:,3) == string(datos_duro(z,1))));
                            pads(index_pincambio,5) = string(-str2double(pads(index_pincambio,5)));
                        end
                    end
                    fprintf(fileID,'%s', new(NLnew(6)-1:NLnew(linea_pads)));                    
                    % Escribimos los pads segun su tipo y forma
                    for z=1:size(pads,1)
                        if strcmp(pads(z,7), 'thru') == 1
                            if strcmp(pads(z,8), 'rnd') == 1
                                fprintf(fileID, '  (pad %s thru_hole circle (at %s %s) (size %s %s) (drill %s) (layers *.Cu *.Mask))\n', pads(z,3),pads(z,4),pads(z,5), pads(z,9),pads(z,10),pads(z,11));
                            elseif strcmp(pads(z,8), 'sq') == 1
                                fprintf(fileID, '  (pad %s thru_hole rect (at %s %s) (size %s %s) (drill %s) (layers *.Cu *.Mask))\n', pads(z,3),pads(z,4),pads(z,5), pads(z,9),pads(z,10),pads(z,11));
                            elseif strcmp(pads(z,8), 'rect') == 1
                                fprintf(fileID, '  (pad %s thru_hole rect (at %s %s) (size %s %s) (drill %s) (layers *.Cu *.Mask))\n', pads(z,3),pads(z,4),pads(z,5), pads(z,9),pads(z,10),pads(z,11));
                            elseif strcmp(pads(z,8), 'oval') == 1
                                fprintf(fileID, '  (pad %s thru_hole oval (at %s %s) (size %s %s) (drill %s) (layers *.Cu *.Mask))\n', pads(z,3),pads(z,4),pads(z,5), pads(z,9),pads(z,10),pads(z,11));
                            end
                        else 
                            if strcmp(pads(z,8), 'rnd') == 1
                                fprintf(fileID, '  (pad %s smd circle (at %s %s) (size %s %s) (layers F.Cu F.Paste F.Mask))\n', pads(z,3),pads(z,4),pads(z,5), pads(z,9),pads(z,10));
                            elseif strcmp(pads(z,8), 'sq') == 1
                                fprintf(fileID, '  (pad %s smd rect (at %s %s) (size %s %s) (layers F.Cu F.Paste F.Mask))\n', pads(z,3),pads(z,4),pads(z,5), pads(z,9),pads(z,10));
                            elseif strcmp(pads(z,8), 'rect') == 1
                                fprintf(fileID, '  (pad %s smd rect (at %s %s) (size %s %s) (layers F.Cu F.Paste F.Mask))\n', pads(z,3),pads(z,4),pads(z,5), pads(z,9),pads(z,10));
                            elseif strcmp(pads(z,8), 'oval') == 1
                                fprintf(fileID, '  (pad %s smd oval (at %s %s) (size %s %s) (layers F.Cu F.Paste F.Mask))\n', pads(z,3),pads(z,4),pads(z,5), pads(z,9),pads(z,10));
                            end
                        end
                    end
                    % Escribimos el resto del modelo
                    fprintf(fileID,'%s', new(NLnew(linea_pads+size(pads,1)):end));
                end
            end
        end
    end
end

% Escribimos el final del archivo        
fprintf(fileID, '\n \n)');
% Cerramos archivo
fclose('all');
% Mensaje de finalizacin
msgbox(msg);

end